@@ -1,26 +1,28 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 
-from xpra.log import Logger
-log = Logger("client")
-workspacelog = Logger("client", "workspace")
-screenlog = Logger("client", "screen")
-scalinglog = Logger("scaling")
-
-
 from xpra.exit_codes import EXIT_INTERNAL_ERROR
 from xpra.platform.features import REINIT_WINDOWS
-from xpra.platform.gui import (get_antialias_info, get_icc_info, get_display_icc_info, show_desktop, get_cursor_size,
-                               get_xdpi, get_ydpi, get_number_of_desktops, get_desktop_names, get_wm_name)
+from xpra.platform.gui import (
+    get_antialias_info, get_icc_info, get_display_icc_info, show_desktop, get_cursor_size,
+    get_xdpi, get_ydpi, get_number_of_desktops, get_desktop_names, get_wm_name,
+    )
 from xpra.scripts.config import FALSE_OPTIONS
 from xpra.os_util import monotonic_time
-from xpra.util import iround, envint, envfloat, log_screen_sizes, engs, flatten_dict, XPRA_SCALING_NOTIFICATION_ID
+from xpra.util import (
+    iround, envint, envfloat, envbool, log_screen_sizes, engs, flatten_dict,
+    XPRA_SCALING_NOTIFICATION_ID,
+    )
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
+from xpra.log import Logger
 
+log = Logger("screen")
+workspacelog = Logger("client", "workspace")
+scalinglog = Logger("scaling")
 
 MONITOR_CHANGE_REINIT = envint("XPRA_MONITOR_CHANGE_REINIT")
 
@@ -29,6 +31,7 @@ MIN_SCALING = envfloat("XPRA_MIN_SCALING", "0.1")
 MAX_SCALING = envfloat("XPRA_MAX_SCALING", "8")
 SCALING_OPTIONS = [float(x) for x in os.environ.get("XPRA_TRAY_SCALING_OPTIONS", "0.25,0.5,0.666,1,1.25,1.5,2.0,3.0,4.0,5.0").split(",") if float(x)>=MIN_SCALING and float(x)<=MAX_SCALING]
 SCALING_EMBARGO_TIME = int(os.environ.get("XPRA_SCALING_EMBARGO_TIME", "1000"))/1000.0
+SYNC_ICC = envbool("XPRA_SYNC_ICC", True)
 
 
 def r4cmp(v, rounding=1000.0):    #ignore small differences in floats for scale values
@@ -54,13 +57,14 @@ class DisplayClient(StubClientMixin):
         self.desktop_scaling = False
         self.screen_size_change_timer = None
 
+        self.server_desktop_size = None
         self.server_actual_desktop_size = None
         self.server_max_desktop_size = None
         self.server_display = None
         self.server_randr = False
 
 
-    def init(self, opts):
+    def init(self, opts, _extra_args=()):
         self.desktop_fullscreen = opts.desktop_fullscreen
         self.desktop_scaling = opts.desktop_scaling
         self.dpi = int(opts.dpi)
@@ -94,13 +98,15 @@ class DisplayClient(StubClientMixin):
             }
         wm_name = get_wm_name()
         if wm_name:
-            caps["wm_name"] = wm_name
+            caps["wm_name"] = wm_name.encode("utf8")
 
         self._last_screen_settings = self.get_screen_settings()
         root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, _, _ = self._last_screen_settings
-        caps["desktop_size"] = self.cp(u_root_w, u_root_h)
-        caps["desktops"] = ndesktops
-        caps["desktop.names"] = desktop_names
+        if u_root_w and u_root_h:
+            caps["desktop_size"] = self.cp(u_root_w, u_root_h)
+        if ndesktops:
+            caps["desktops"] = ndesktops
+            caps["desktop.names"] = tuple(x.encode("utf8") for x in desktop_names)
 
         ss = self.get_screen_sizes()
         self._current_screen_sizes = ss
@@ -115,7 +121,9 @@ class DisplayClient(StubClientMixin):
                 sinfo = "%i%%" % iround(self.xscale*100)
             else:
                 sinfo = "%i%% x %i%%" % (iround(self.xscale*100), iround(self.yscale*100))
-            log.info(" %sscaled by %s, virtual screen size: %ix%i", ["down", "up"][int(u_root_w>root_w or u_root_h>root_h)], sinfo, root_w, root_h)
+            scaled_up = u_root_w>root_w or u_root_h>root_h
+            log.info(" %sscaled to %s, virtual screen size: %ix%i",
+                     "up" if scaled_up else "down", sinfo, root_w, root_h)
             log_screen_sizes(root_w, root_h, sss)
         else:
             root_w, root_h = u_root_w, u_root_h
@@ -135,13 +143,13 @@ class DisplayClient(StubClientMixin):
         dpi = 0
         if self.dpi>0:
             #scale it:
-            xdpi = ydpi = dpi = self.cx(self.cy(self.dpi))
+            dpi = iround((self.cx(self.dpi) + self.cy(self.dpi))/2.0)
         else:
             #not supplied, use platform detection code:
             #platforms may also provide per-axis dpi (later win32 versions do)
             xdpi = self.get_xdpi()
             ydpi = self.get_ydpi()
-            screenlog("xdpi=%i, ydpi=%i", xdpi, ydpi)
+            log("xdpi=%i, ydpi=%i", xdpi, ydpi)
             if xdpi>0 and ydpi>0:
                 xdpi = self.cx(xdpi)
                 ydpi = self.cy(ydpi)
@@ -152,7 +160,7 @@ class DisplayClient(StubClientMixin):
                     }
         if dpi:
             caps[""] = dpi
-        screenlog("dpi: %i", dpi)
+        log("get_gpi_caps()=%s", caps)
         return caps
 
     def get_scaling_caps(self):
@@ -163,14 +171,18 @@ class DisplayClient(StubClientMixin):
             }
 
     def get_screen_caps(self):
-        return {
+        caps = {
             "antialias"    : get_antialias_info(),
-            "icc"          : self.get_icc_info(),
-            "display-icc"  : self.get_display_icc_info(),
             "cursor" : {
                 "size"  : int(2*get_cursor_size()/(self.xscale+self.yscale)),
                 },
             }
+        if SYNC_ICC:
+            caps.update({
+            "icc"          : self.get_icc_info(),
+            "display-icc"  : self.get_display_icc_info(),
+            })
+        return caps
 
     #this is the format we should be moving towards
     #with proper namespace:
@@ -202,6 +214,8 @@ class DisplayClient(StubClientMixin):
     def parse_server_capabilities(self):
         c = self.server_capabilities
         self.server_display = c.strget("display")
+        self.server_desktop_size = c.intpair("desktop_size")
+        log("server desktop size=%s", self.server_desktop_size)
         self.server_max_desktop_size = c.intpair("max_desktop_size")
         self.server_actual_desktop_size = c.intpair("actual_desktop_size")
         log("server actual desktop size=%s", self.server_actual_desktop_size)
@@ -211,8 +225,6 @@ class DisplayClient(StubClientMixin):
 
     def process_ui_capabilities(self):
         c = self.server_capabilities
-        server_desktop_size = c.intlistget("desktop_size")
-        log("server desktop size=%s", server_desktop_size)
         self.server_is_desktop = c.boolget("shadow") or c.boolget("desktop")
         skip_vfb_size_check = False           #if we decide not to use scaling, skip warnings
         if not fequ(self.xscale, 1.0) or not fequ(self.yscale, 1.0):
@@ -237,7 +249,8 @@ class DisplayClient(StubClientMixin):
         if not self.server_is_desktop and not skip_vfb_size_check and self.server_max_desktop_size:
             avail_w, avail_h = self.server_max_desktop_size
             root_w, root_h = self.get_root_size()
-            log("validating server_max_desktop_size=%s vs root size %s scaled to %s", self.server_max_desktop_size, (root_w, root_h), (self.cx(root_w), self.cy(root_h)))
+            log("validating server_max_desktop_size=%s vs root size %s scaled to %s",
+                self.server_max_desktop_size, (root_w, root_h), (self.cx(root_w), self.cy(root_h)))
             if self.cx(root_w)>(avail_w+1) or self.cy(root_h)>(avail_h+1):
                 log.warn("Server's virtual screen is too small")
                 log.warn(" server: %sx%s vs client: %sx%s", avail_w, avail_h, self.cx(root_w), self.cy(root_h))
@@ -254,7 +267,7 @@ class DisplayClient(StubClientMixin):
             server_w, server_h = self.server_actual_desktop_size
             maxw = max(root_w, server_w)
             maxh = max(root_h, server_h)
-        except:
+        except ValueError:
             pass
         if maxw<=0 or maxh<=0 or maxw>=32768 or maxh>=32768:
             message = "invalid maximum desktop size: %ix%i" % (maxw, maxh)
@@ -269,8 +282,10 @@ class DisplayClient(StubClientMixin):
         # both with enough headroom for some metadata (4k)
         p = self._protocol
         if p:
-            p.max_packet_size = max(maxw*maxh*4, self.file_size_limit*1024*1024) + 4*1024
-            p.abs_max_packet_size = max(maxw*maxh*4 * 4, self.file_size_limit*1024*1024) + 4*1024
+            #we can't assume to have a real ClientConnection object:
+            file_size_limit = getattr(self, "file_size_limit", 0)
+            p.max_packet_size = max(maxw*maxh*4, file_size_limit*1024*1024) + 4*1024
+            p.abs_max_packet_size = max(maxw*maxh*4 * 4, file_size_limit*1024*1024) + 4*1024
             log("maximum packet size set to %i", p.max_packet_size)
 
 
@@ -290,7 +305,7 @@ class DisplayClient(StubClientMixin):
 
     def _process_desktop_size(self, packet):
         root_w, root_h, max_w, max_h = packet[1:5]
-        screenlog("server has resized the desktop to: %sx%s (max %sx%s)", root_w, root_h, max_w, max_h)
+        log("server has resized the desktop to: %sx%s (max %sx%s)", root_w, root_h, max_w, max_h)
         self.server_max_desktop_size = max_w, max_h
         self.server_actual_desktop_size = root_w, root_h
         if self.can_scale:
@@ -298,7 +313,8 @@ class DisplayClient(StubClientMixin):
 
 
     def may_adjust_scaling(self):
-        log("may_adjust_scaling() server_is_desktop=%s, desktop_fullscreen=%s", self.server_is_desktop, self.desktop_fullscreen)
+        log("may_adjust_scaling() server_is_desktop=%s, desktop_fullscreen=%s",
+            self.server_is_desktop, self.desktop_fullscreen)
         if self.server_is_desktop and not self.desktop_fullscreen:
             #don't try to make it fit
             return
@@ -306,26 +322,35 @@ class DisplayClient(StubClientMixin):
         max_w, max_h = self.server_max_desktop_size             #ie: server limited to 8192x4096?
         w, h = self.get_root_size()                             #ie: 5760, 2160
         sw, sh = self.cp(w, h)                                  #ie: upscaled to: 11520x4320
-        scalinglog("may_adjust_scaling() server desktop size=%s, client root size=%s", self.server_actual_desktop_size, self.get_root_size())
+        scalinglog("may_adjust_scaling() server desktop size=%s, client root size=%s",
+                   self.server_actual_desktop_size, self.get_root_size())
         scalinglog(" scaled client root size using %sx%s: %s", self.xscale, self.yscale, (sw, sh))
-        if sw<(max_w+1) and sh<(max_h+1):
-            #no change needed
-            return
         #server size is too small for the client screen size with the current scaling value,
         #calculate the minimum scaling to fit it:
         def clamp(v):
             return max(MIN_SCALING, min(MAX_SCALING, v))
-        x = clamp(float(w)/max_w)
-        y = clamp(float(h)/max_h)
+        if self.desktop_fullscreen:
+            sw, sh = self.server_actual_desktop_size
+            x = clamp(float(w)/sw)
+            y = clamp(float(h)/sh)
+        else:
+            if sw<(max_w+1) and sh<(max_h+1):
+                #no change needed
+                return
+            x = clamp(float(w)/max_w)
+            y = clamp(float(h)/max_h)
+            #avoid wonky scaling:
+            if not 0.75<x/y<1.25:
+                x = y = min(x, y)
         def mint(v):
             #prefer int over float,
             #and even tolerate a 0.1% difference to get it:
             if iround(v)*1000==iround(v*1000):
                 return int(v)
             return v
-        mscale = max(mint(x), mint(y))
-        self.xscale = mscale
-        self.yscale = mscale
+        self.xscale = mint(x)
+        self.yscale = mint(y)
+        scalinglog(" xscale=%s, yscale=%s", self.xscale, self.yscale)
         #to use the same scale for both axes:
         #self.xscale = mint(max(x, y))
         #self.yscale = self.xscale
@@ -345,12 +370,18 @@ class DisplayClient(StubClientMixin):
 
     ######################################################################
     # screen scaling:
+    def fsx(self, v):
+        """ convert X coordinate from server to client """
+        return v*self.xscale
+    def fsy(self, v):
+        """ convert Y coordinate from server to client """
+        return v*self.yscale
     def sx(self, v):
         """ convert X coordinate from server to client """
-        return iround(v*self.xscale)
+        return iround(self.fsx(v))
     def sy(self, v):
         """ convert Y coordinate from server to client """
-        return iround(v*self.yscale)
+        return iround(self.fsy(v))
     def srect(self, x, y, w, h):
         """ convert rectangle coordinates from server to client """
         return self.sx(x), self.sy(y), self.sx(w), self.sy(h)
@@ -384,7 +415,7 @@ class DisplayClient(StubClientMixin):
             win.workspace_changed()
 
     def screen_size_changed(self, *args):
-        screenlog("screen_size_changed(%s) timer=%s", args, self.screen_size_change_timer)
+        log("screen_size_changed(%s) timer=%s", args, self.screen_size_change_timer)
         if self.screen_size_change_timer:
             return
         #update via timer so the data is more likely to be final (up to date) when we query it,
@@ -401,11 +432,12 @@ class DisplayClient(StubClientMixin):
     def do_process_screen_size_change(self):
         self.screen_size_change_timer = None
         self.update_screen_size()
-        screenlog("do_process_screen_size_change() MONITOR_CHANGE_REINIT=%s, REINIT_WINDOWS=%s", MONITOR_CHANGE_REINIT, REINIT_WINDOWS)
+        log("do_process_screen_size_change() MONITOR_CHANGE_REINIT=%s, REINIT_WINDOWS=%s",
+            MONITOR_CHANGE_REINIT, REINIT_WINDOWS)
         if MONITOR_CHANGE_REINIT and MONITOR_CHANGE_REINIT=="0":
             return
         if MONITOR_CHANGE_REINIT or REINIT_WINDOWS:
-            screenlog.info("screen size change: will reinit the windows")
+            log.info("screen size change: will reinit the windows")
             self.reinit_windows()
             self.reinit_window_icons()
 
@@ -417,7 +449,7 @@ class DisplayClient(StubClientMixin):
         sss = self.get_screen_sizes(self.xscale, self.yscale)
         ndesktops = get_number_of_desktops()
         desktop_names = get_desktop_names()
-        screenlog("update_screen_size() sizes=%s, %s desktops: %s", sss, ndesktops, desktop_names)
+        log("update_screen_size() sizes=%s, %s desktops: %s", sss, ndesktops, desktop_names)
         if self.dpi>0:
             #use command line value supplied, but scale it:
             xdpi = ydpi = self.dpi
@@ -427,21 +459,22 @@ class DisplayClient(StubClientMixin):
             ydpi = self.get_ydpi()
         xdpi = self.cx(xdpi)
         ydpi = self.cy(ydpi)
-        screenlog("dpi: %s -> %s", (get_xdpi(), get_ydpi()), (xdpi, ydpi))
+        log("dpi: %s -> %s", (get_xdpi(), get_ydpi()), (xdpi, ydpi))
         return (root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, xdpi, ydpi)
 
     def update_screen_size(self):
         self.screen_size_change_timer = None
         screen_settings = self.get_screen_settings()
-        screenlog("update_screen_size()     new settings=%s", screen_settings)
-        screenlog("update_screen_size() current settings=%s", self._last_screen_settings)
+        log("update_screen_size()     new settings=%s", screen_settings)
+        log("update_screen_size() current settings=%s", self._last_screen_settings)
         if self._last_screen_settings==screen_settings:
             log("screen size unchanged")
             return
         root_w, root_h, sss = screen_settings[:3]
-        screenlog.info("sending updated screen size to server: %sx%s with %s screens", root_w, root_h, len(sss))
+        log.info("sending updated screen size to server: %sx%s with %s screens", root_w, root_h, len(sss))
         log_screen_sizes(root_w, root_h, sss)
-        self.send("desktop_size", *screen_settings)
+        if self.server_desktop_size:
+            self.send("desktop_size", *screen_settings)
         self._last_screen_settings = screen_settings
         #update the max packet size (may have gone up):
         self.set_max_packet_size()
@@ -495,7 +528,8 @@ class DisplayClient(StubClientMixin):
             scalinglog("scale_change(%s, %s) screen size change is already pending", xchange, ychange)
             return
         if monotonic_time()<self.scale_change_embargo:
-            scalinglog("scale_change(%s, %s) screen size change not permitted during embargo time - try again", xchange, ychange)
+            scalinglog("scale_change(%s, %s) screen size change not permitted during embargo time - try again",
+                       xchange, ychange)
             return
         def clamp(v):
             return max(MIN_SCALING, min(MAX_SCALING, v))
@@ -524,7 +558,7 @@ class DisplayClient(StubClientMixin):
                 "the scaled client screen %i x %i -> %i x %i" % (root_w, root_h, sw, sh),
                 " would overflow the server's screen: %i x %i" % (maxw, maxh),
                 ]
-            self.may_notify(XPRA_SCALING_NOTIFICATION_ID, summary, "\n".join(messages), "scaling")
+            self.may_notify(XPRA_SCALING_NOTIFICATION_ID, summary, "\n".join(messages), icon_name="scaling")
             scalinglog.warn("Warning: %s", summary)
             for m in messages:
                 scalinglog.warn(" %s", m)
@@ -547,14 +581,12 @@ class DisplayClient(StubClientMixin):
             minx, miny = 16384, 16384
             if self.max_window_size!=(0, 0):
                 minx, miny = self.max_window_size
-            return max(1, min(minx, int(w*xchange))), max(1, min(miny, int(h*ychange)))
-        self.reinit_windows(new_size_fn)
+            return max(1, min(minx, iround(w*xchange))), max(1, min(miny, iround(h*ychange)))
+        self.resize_windows(new_size_fn)
         self.reinit_window_icons()
         self.emit("scaling-changed")
 
 
     def init_authenticated_packet_handlers(self):
-        self.set_packet_handlers(self._ui_packet_handlers, {
-            "show-desktop":         self._process_show_desktop,
-            "desktop_size":         self._process_desktop_size,
-            })
+        self.add_packet_handler("show-desktop", self._process_show_desktop)
+        self.add_packet_handler("desktop_size", self._process_desktop_size)
