@@ -1,13 +1,13 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+from collections import deque
 
 from xpra.platform.dotxpra import DotXpra
-from xpra.util import envint
-from collections import deque
+from xpra.util import envint, obsc
 from xpra.net.digest import get_salt, choose_digest, verify_digest, gendigest
 from xpra.os_util import hexstr, POSIX
 from xpra.log import Logger
@@ -30,7 +30,7 @@ def parse_uid(v):
     if v:
         try:
             return int(v)
-        except:
+        except (TypeError, ValueError):
             log("uid '%s' is not an int", v)
     if POSIX:
         try:
@@ -46,7 +46,7 @@ def parse_gid(v):
     if v:
         try:
             return int(v)
-        except:
+        except (TypeError, ValueError):
             log("gid '%s' is not an int", v)
     if POSIX:
         try:
@@ -96,7 +96,7 @@ class SysAuthenticatorBase(object):
         return self.salt, self.digest
 
     def get_passwords(self):
-        p = self.get_password()
+        p = self.get_password()     #pylint: disable=assignment-from-none
         if p is not None:
             return (p,)
         return ()
@@ -104,8 +104,8 @@ class SysAuthenticatorBase(object):
     def get_password(self):
         return None
 
-    def check(self, password):
-        raise NotImplementedError()
+    def check(self, _password):
+        return False
 
     def authenticate(self, challenge_response=None, client_salt=None):
         #this will call check(password)
@@ -135,14 +135,17 @@ class SysAuthenticatorBase(object):
             return False
         salt = self.get_response_salt(client_salt)
         password = gendigest("xor", challenge_response, salt)
-        log("authenticate_check(%s, %s) xor(%s)=%s", repr(challenge_response), repr(client_salt), repr(salt), repr(password))
+        log("authenticate_check(%s, %s) xor(%s)=%s",
+            obsc(repr(challenge_response)), repr(client_salt), repr(salt), repr(password))
         #warning: enabling logging here would log the actual system password!
-        #log.info("authenticate(%s, %s) password=%s (%s)", hexstr(challenge_response), hexstr(client_salt), password, hexstr(password))
+        #log.info("authenticate(%s, %s) password=%s (%s)",
+        #    hexstr(challenge_response), hexstr(client_salt), password, hexstr(password))
         #verify login:
         try :
             ret = self.check(password)
             log("authenticate_check(..)=%s", ret)
         except Exception as e:
+            log("check(..)", exc_info=True)
             log.error("Error: %s authentication check failed:", self)
             log.error(" %s", e)
             return False
@@ -194,12 +197,13 @@ class SysAuthenticator(SysAuthenticatorBase):
 
     def __init__(self, username, **kwargs):
         SysAuthenticatorBase.__init__(self, username)
-        try:
-            import pwd
-            self.pw = pwd.getpwnam(username)
-        except Exception:
-            log("cannot load password database entry for '%s'", username, exc_info=True)
-            self.pw = None
+        self.pw = None
+        if POSIX:
+            try:
+                import pwd
+                self.pw = pwd.getpwnam(username)
+            except Exception:
+                log("cannot load password database entry for '%s'", username, exc_info=True)
 
     def get_uid(self):
         if self.pw is None:
